@@ -1,11 +1,11 @@
 from django import forms
-from django.forms.formsets import formset_factory
 from django.core import validators
 from django.utils.datastructures import SortedDict
 from itertools import chain
 from django.utils.encoding import StrAndUnicode, force_unicode
 from django.utils.html import escape, conditional_escape
 from django.utils.safestring import mark_safe
+from django.forms import widgets
 
 from tiote import utils
 
@@ -43,11 +43,13 @@ export_choices = ( ('structure', 'structure'),('data', 'data') )
 
 foreign_key_action_choices = ['no action', 'restrict', 'cascade', 'set null', 'set default']
 
-def b(cond):
-    if cond == 'NO' or cond == False or cond == 'no': return "NOT NULL"
-    else: return ""
 
-class wCheckboxSelectMultiple(forms.SelectMultiple):
+class tt_CheckboxSelectMultiple(widgets.CheckboxSelectMultiple):
+    """
+    Copy of that found in stock django but added here in other to change its rendering (
+    addition of a class to part of its rendered html)
+    """
+
     def render(self, name, value, attrs=None, choices=()):
         if value is None: value = []
         has_id = attrs and 'id' in attrs
@@ -72,20 +74,41 @@ class wCheckboxSelectMultiple(forms.SelectMultiple):
         output.append(u'</ul>')
         return mark_safe(u'\n'.join(output))
 
+
+class tt_RadioFieldRenderer(widgets.RadioFieldRenderer):
+    """
+    Copy of that found in stock django but added here in other to change its rendering (
+    addition of a class to part of its rendered html)
+    """
+
+    def render(self):
+        """Outputs a <ul> for this set of radio fields."""
+        return mark_safe(u'<ul class="inputs-list">\n%s\n</ul>' % u'\n'.join([u'<li>%s</li>'
+                % force_unicode(w) for w in self]))
+
+
 class InsertForm(forms.BaseForm):
+    '''
+    Dynamically created form which generates its fields along with the fields' options
+    from its parameters. 
+
+    Does not make use of metaclasses so it subclasses forms.BaseForm directly.
+
+    It loops through the parameter ``tbl_struct``(the structure of a table) and then generates
+    fiels which would fit the description of respective columns
+
+    It also treats some fields specially as defined in ``tbl_indexes`` and the form's ``dialect``
+    '''
+
     def __init__(self, dialect, tbl_struct, tbl_indexes=(), **kwargs):
 # keys = ['column','type','null','default','character_maximum_length','numeric_precision', 'extra','column_type']
         f = SortedDict()
         # dict to increase performance
-        _l = {}
-        for i in range(len(tbl_indexes)):
-            if _l.has_key(tbl_indexes[i][0]): _l[ tbl_indexes[i][0] ].append(i)
-            else: _l[ tbl_indexes[i][0] ] = [i]
+        indexed_cols = utils.fns.parse_indexes_query(tbl_indexes)
+        
         # determing type of form fields for each column
         for row in tbl_struct['rows']:
-            _c = []
-            #required fields
-            if b(row[2]): _c.append("required")
+            _classes = []
 
             if row[1] in ('character varying', 'varchar','character', 'char'):
                 f[row[0]] = forms.CharField()
@@ -104,65 +127,106 @@ class InsertForm(forms.BaseForm):
             elif row[1] in ('tinyint', 'smallint', 'mediumint', 'int', 'bigint','integer',):
                 f[row[0]] = forms.IntegerField()
                 # if row[5]: f[row[0]].validators.append(validators.MaxLengthValidator(row[5]))
-                _c.append('validate-integer')
+                _classes.append('validate-integer')
                 
             elif row[1] in ('real', 'double', 'float', 'decimal', 'numeric', 'double precision'):
                 f[row[0]] = forms.FloatField()
                 # if row[5]: f[row[0]].validators.append(validators.MaxLengthValidator(row[5]))
-                _c.append('validate-numeric')
+                _classes.append('validate-numeric')
 
             elif row[1] in ('decimal', 'numeric', 'money',):
                 f[row[0]] = forms.DecimalField()
                 # if row[5]: f[row[0]].validators.append(validators.MaxLengthValidator(row[5]))
-                _c.append('validate-numeric')
+                _classes.append('validate-numeric')
 
             elif row[1] in ('date',):
                 f[row[0]] = forms.DateField()
-                _c.append('validate-date')
-
-            elif row[1] in ('time','time with time zone',):
-                f[row[0]] = forms.TimeField()
-
-            elif row[1] in ('datetime', 'timestamp', 'timestamp with time zone',):
-                f[row[0]] = forms.DateTimeField()
+                _classes.append('validate-date')
 
+            elif row[1] in ('datetime', 'time','time with time zone','timestamp', 'timestamp with time zone',):
+                # no longer used a python field (date, datetime) because
+                # - of error generated when submitting fields which
+                # - are populated from the database
+                f[row[0]] = forms.CharField() 
+                
             elif row[1] == 'set':
-                f[row[0]] = forms.ChoiceField(widget=wCheckboxSelectMultiple())
-                f[row[0]].choices = utils.fns.make_choices(
-                    row[len(row)-1].replace("set(", "").replace(")","").split(","), True) 
+                f[row[0]] = forms.MultipleChoiceField(widget=tt_CheckboxSelectMultiple())
+                # parse the field description to list with all the unnecssary quotes removed
+                choices = row[len(row)-1].replace("set(", "").replace(")","")
+                choices = choices.replace("'", "").split(",")
+                f[row[0]].choices = utils.fns.make_choices(choices, True) 
 
             elif row[1] == 'enum':
                 f[row[0]] = forms.ChoiceField()
-                f[row[0]].choices = utils.fns.make_choices(
-                    row[len(row)-1].replace("enum(", "").replace("\"","").replace(")","").split(","), False) 
-            # any field not currently understood
+                # parse the field description to list with all the unnecssary quotes removed
+                choices = row[len(row)-1].replace("enum(", "").replace("\"","").replace(")","")
+                choices = choices.replace("'", "").split(",")
+                f[row[0]].choices = utils.fns.make_choices(choices, False) 
+            
+            # any field not currently understood (PostgreSQL makes use of a lot of user defined fields
+                # which is difficult to keep track of)
             else: f[row[0]] = forms.CharField(widget=forms.Textarea(attrs={'cols':'', 'rows':''}))
 
+            #required fields
+            if row[2].lower() == 'no' or row[2] == False:
+                # the field row[2] is required
+                _classes.append("required")
+                # the option  above must be the last assignment to _classes because it's index
+                # - must be the last one for the next lines of logic to work
+            else:
+                f[row[0]].required = False
+                
             # options common to all fields
             # help_text
             _il = [ row[len(row) - 1], ]
             if dialect == 'mysql': _il.append(row[len(row) -2 ])
             f[row[0]].help_text =  " ".join(_il)
             if row[3]: f[row[0]].default = row[3] #default
+            
             # work with indexes
-            if _l.has_key( row[0] ):
-                for i in _l[row[0]]:
-                    if dialect == 'mysql' and tbl_indexes[i][2] == "PRIMARY KEY":
-                        if row[len(row) - 2].count('auto_increment'): _c.pop(0) # make it not required
+            if indexed_cols.has_key( row[0] ):
+                if dialect == 'mysql' and indexed_cols[ row[0] ].count("PRIMARY KEY"):
+                    # make an indexed column with auto_increment flag not required (MySQL)
+                    if row[len(row) - 2].count('auto_increment') > 0: 
+                        if _classes.count('required') > 0: _classes.pop()
+                        f[ row[0] ].required = False
 
             # width of the fields
-            if type(f[row[0]].widget) not in (forms.CheckboxSelectMultiple, wCheckboxSelectMultiple,):
-                _c.append("span6")
+            if type(f[row[0]].widget) not in (forms.CheckboxSelectMultiple, tt_CheckboxSelectMultiple,):
+                _classes.append("span6")
             # add the attribute classes                
             if f[row[0]].widget.attrs.has_key('class'):
-                f[row[0]].widget.attrs['class'] += " ".join(_c)
+                f[row[0]].widget.attrs['class'] += " ".join(_classes)
             else:
-                f[row[0]].widget.attrs.update({'class':" ".join(_c)})
+                f[row[0]].widget.attrs.update({'class':" ".join(_classes)})
 
 
         self.base_fields = f
         forms.BaseForm.__init__(self, **kwargs)
-    
+
+
+class EditForm(InsertForm):
+    '''
+    Subclasses InsertForm to include the dynamic property of InsertForm as well as to 
+    add and option that specifies if the request would be for a new row or would be 
+    an update for that row
+    '''
+
+    def __init__(self, dialect, tbl_struct, tbl_indexes=(), **kwargs):
+        InsertForm.__init__(self, dialect, tbl_struct, tbl_indexes, **kwargs)
+
+        # working with self.fields attribute because this is an instance of InsertForm
+        # - and not a whole form class definition
+        self.fields['save_changes_to'] = forms.ChoiceField(
+            label = 'save changes to',
+            choices = (('update_row', 'Same row (UPDATE statment)',),
+                ('insert_row', 'Another row (INSERT statement)')
+            ), 
+            initial = 'update_row',
+            widget = forms.RadioSelect(attrs={'class':'inputs-list'}, renderer = tt_RadioFieldRenderer)
+        )
+
+
 # New Database Form
 class mysqlDbForm(forms.Form):
     def __init__(self, templates=None, users=None, charsets=None, **kwargs):
@@ -515,7 +579,8 @@ class pgsqlSequenceForm(forms.Form):
 class QueryForm(forms.Form):
     query = forms.CharField(label = u"Enter your query:", 
         widget = forms.Textarea(attrs={'class':'required span10','rols':0, 'cols':0, 'style':'height:100px;resize:none;'},) )
-    
+
+
 def get_dialect_form(form_name, dialect):
     '''
     structure of dialect_forms:
